@@ -56,9 +56,11 @@
 #define pixman_region_contains_point _cairo_pixman_region_contains_point
 #define pixman_region_contains_rectangle _cairo_pixman_region_contains_rectangle
 #define pixman_region_copy _cairo_pixman_region_copy
-#define pixman_region_create _cairo_pixman_region_create
-#define pixman_region_create_simple _cairo_pixman_region_create_simple
-#define pixman_region_destroy _cairo_pixman_region_destroy
+#define pixman_region_init _cairo_pixman_region_init
+#define pixman_region_init_rect _cairo_pixman_region_init_rect
+#define pixman_region_init_rects _cairo_pixman_region_init_rects
+#define pixman_region_init_with_extents _cairo_pixman_region_init_with_extents
+#define pixman_region_fini _cairo_pixman_region_fini
 #define pixman_region_empty _cairo_pixman_region_empty
 #define pixman_region_extents _cairo_pixman_region_extents
 #define pixman_region_intersect _cairo_pixman_region_intersect
@@ -77,4 +79,3 @@
 #define RenderLineFixedEdgeInit _cairo_pixman_render_line_fixed_edge_init
 #define RenderSampleCeilY _cairo_pixman_render_sample_ceil_y
 #define RenderSampleFloorY _cairo_pixman_render_sample_floor_y
-#define fbSolidFillmmx _cairo_pixman_solid_fill_mmx
