@@ -26,13 +26,15 @@
 
 #include "cairoint.h"
 #include "cairo-glitz.h"
+#include "cairo-glitz-private.h"
 
 typedef struct _cairo_glitz_surface {
     cairo_surface_t   base;
 
     glitz_surface_t   *surface;
     glitz_format_t    *format;
-    pixman_region16_t *clip;
+    cairo_bool_t      has_clip;
+    pixman_region16_t clip;
 } cairo_glitz_surface_t;
 
 static const cairo_surface_backend_t *
@@ -43,10 +45,9 @@ _cairo_glitz_surface_finish (void *abstract_surface)
 {
     cairo_glitz_surface_t *surface = abstract_surface;
 
-    if (surface->clip)
-    {
-	glitz_surface_set_clip_region (surface->surface, 0, 0, NULL, 0);
-	pixman_region_destroy (surface->clip);
+    if (surface->has_clip) {
+        glitz_surface_set_clip_region (surface->surface, 0, 0, NULL, 0);
+        pixman_region_fini (&surface->clip);
     }
 
     glitz_surface_destroy (surface->surface);
@@ -207,8 +208,9 @@ _cairo_glitz_surface_get_image (cairo_glitz_surface_t   *surface,
     }
 
     /* clear out the glitz clip; the clip affects glitz_get_pixels */
-    glitz_surface_set_clip_region (surface->surface,
-				   0, 0, NULL, 0);
+    if (surface->has_clip)
+	glitz_surface_set_clip_region (surface->surface,
+				       0, 0, NULL, 0);
 
     glitz_get_pixels (surface->surface,
 		      x1, y1,
@@ -219,8 +221,14 @@ _cairo_glitz_surface_get_image (cairo_glitz_surface_t   *surface,
     glitz_buffer_destroy (buffer);
 
     /* restore the clip, if any */
-    surface->base.current_clip_serial = 0;
-    _cairo_surface_set_clip (&surface->base, surface->base.clip);
+    if (surface->has_clip) {
+	glitz_box_t *box;
+	int	    n;
+
+	box = (glitz_box_t *) pixman_region_rects (&surface->clip);
+	n = pixman_region_num_rects (&surface->clip);
+	glitz_surface_set_clip_region (surface->surface, 0, 0, box, n);
+    }
 
     image = (cairo_image_surface_t *)
 	_cairo_image_surface_create_with_masks (pixels,
@@ -243,29 +251,34 @@ _cairo_glitz_surface_get_image (cairo_glitz_surface_t   *surface,
 static cairo_status_t
 _cairo_glitz_surface_set_image (void		      *abstract_surface,
 				cairo_image_surface_t *image,
-				int		      x_dst,
-				int		      y_dst)
+				int                    src_x,
+				int                    src_y,
+				int                    width,
+				int                    height,
+				int		       x_dst,
+				int		       y_dst)
 {
     cairo_glitz_surface_t *surface = abstract_surface;
     glitz_buffer_t	  *buffer;
     glitz_pixel_format_t  pf;
     pixman_format_t	  *format;
-    int			  am, rm, gm, bm;
+    unsigned int	  bpp, am, rm, gm, bm;
     char		  *data;
 
     format = pixman_image_get_format (image->pixman_image);
     if (!format)
 	return CAIRO_STATUS_NO_MEMORY;
 
-    pixman_format_get_masks (format, &pf.masks.bpp, &am, &rm, &gm, &bm);
+    pixman_format_get_masks (format, &bpp, &am, &rm, &gm, &bm);
 
     pf.fourcc = GLITZ_FOURCC_RGB;
+    pf.masks.bpp = bpp;
     pf.masks.alpha_mask = am;
     pf.masks.red_mask = rm;
     pf.masks.green_mask = gm;
     pf.masks.blue_mask = bm;
-    pf.xoffset = 0;
-    pf.skip_lines = 0;
+    pf.xoffset = src_x;
+    pf.skip_lines = src_y;
 
     /* check for negative stride */
     if (image->stride < 0)
@@ -287,7 +300,7 @@ _cairo_glitz_surface_set_image (void		      *abstract_surface,
 
     glitz_set_pixels (surface->surface,
 		      x_dst, y_dst,
-		      image->width, image->height,
+		      width, height,
 		      &pf,
 		      buffer);
 
@@ -347,7 +360,8 @@ _cairo_glitz_surface_release_dest_image (void                    *abstract_surfa
 {
     cairo_glitz_surface_t *surface = abstract_surface;
 
-    _cairo_glitz_surface_set_image (surface, image,
+    _cairo_glitz_surface_set_image (surface, image, 0, 0,
+				    image->width, image->height,
 				    image_rect->x, image_rect->y);
 
     cairo_surface_destroy (&image->base);
@@ -356,6 +370,10 @@ _cairo_glitz_surface_release_dest_image (void                    *abstract_surfa
 static cairo_status_t
 _cairo_glitz_surface_clone_similar (void	    *abstract_surface,
 				    cairo_surface_t *src,
+				    int              src_x,
+				    int              src_y,
+				    int              width,
+				    int              height,
 				    cairo_surface_t **clone_out)
 {
     cairo_glitz_surface_t *surface = abstract_surface;
@@ -374,6 +392,8 @@ _cairo_glitz_surface_clone_similar (void	    *abstract_surface,
     {
 	cairo_image_surface_t *image_src = (cairo_image_surface_t *) src;
 	cairo_content_t	      content;
+	cairo_rectangle_int16_t image_extent;
+	cairo_rectangle_int16_t extent;
 
 	content = _cairo_content_from_format (image_src->format);
 
@@ -384,7 +404,21 @@ _cairo_glitz_surface_clone_similar (void	    *abstract_surface,
 	if (clone->base.status)
 	    return CAIRO_STATUS_NO_MEMORY;
 
-	_cairo_glitz_surface_set_image (clone, image_src, 0, 0);
+	image_extent.x = 0;
+	image_extent.y = 0;
+	image_extent.width = image_src->width;
+	image_extent.height = image_src->height;
+	extent.x = src_x;
+	extent.y = src_y;
+	extent.width = width;
+	extent.height = height;
+
+	_cairo_rectangle_intersect(&extent, &image_extent);
+
+	_cairo_glitz_surface_set_image (clone, image_src,
+					extent.x, extent.y,
+					extent.width, extent.height,
+					extent.x, extent.y);
 
 	*clone_out = &clone->base;
 
@@ -558,9 +592,9 @@ _cairo_glitz_pattern_acquire_surface (cairo_pattern_t	              *pattern,
 	glitz_fixed16_16_t	    *params;
 	int			    n_params;
 	unsigned int		    *pixels;
-	int			    i, n_base_params;
+	unsigned int		    i, n_base_params;
 	glitz_buffer_t		    *buffer;
-	static glitz_pixel_format_t format = {
+	static const glitz_pixel_format_t format = {
 	    GLITZ_FOURCC_RGB,
 	    {
 		32,
@@ -634,7 +668,7 @@ _cairo_glitz_pattern_acquire_surface (cairo_pattern_t	              *pattern,
 	}
 
 	glitz_set_pixels (src->surface, 0, 0, gradient->n_stops, 1,
-			  &format, buffer);
+			  (glitz_pixel_format_t *)&format, buffer);
 
 	glitz_buffer_destroy (buffer);
 
@@ -652,12 +686,12 @@ _cairo_glitz_pattern_acquire_surface (cairo_pattern_t	              *pattern,
 	{
 	    cairo_radial_pattern_t *grad = (cairo_radial_pattern_t *) pattern;
 
-	    params[0] = grad->gradient.inner.x;
-	    params[1] = grad->gradient.inner.y;
-	    params[2] = grad->gradient.inner.radius;
-	    params[3] = grad->gradient.outer.x;
-	    params[4] = grad->gradient.outer.y;
-	    params[5] = grad->gradient.outer.radius;
+	    params[0] = grad->gradient.c1.x;
+	    params[1] = grad->gradient.c1.y;
+	    params[2] = grad->gradient.c1.radius;
+	    params[3] = grad->gradient.c2.x;
+	    params[4] = grad->gradient.c2.y;
+	    params[5] = grad->gradient.c2.radius;
 	    attr->filter = GLITZ_FILTER_RADIAL_GRADIENT;
 	}
 
@@ -682,6 +716,8 @@ _cairo_glitz_pattern_acquire_surface (cairo_pattern_t	              *pattern,
 	attr->base.x_offset = 0;
 	attr->base.y_offset = 0;
     } break;
+    case CAIRO_PATTERN_TYPE_SOLID:
+    case CAIRO_PATTERN_TYPE_SURFACE:
     default:
 	break;
     }
@@ -723,6 +759,7 @@ _cairo_glitz_pattern_acquire_surface (cairo_pattern_t	              *pattern,
 	    case CAIRO_FILTER_GOOD:
 	    case CAIRO_FILTER_BEST:
 	    case CAIRO_FILTER_BILINEAR:
+	    case CAIRO_FILTER_GAUSSIAN:
 	    default:
 		attr->filter = GLITZ_FILTER_BILINEAR;
 		break;
@@ -785,11 +822,16 @@ _cairo_glitz_pattern_acquire_surfaces (cairo_pattern_t	                *src,
 	combined = src_solid->color;
 	_cairo_color_multiply_alpha (&combined, mask_solid->color.alpha);
 
-	_cairo_pattern_init_solid (&tmp.solid, &combined);
+	_cairo_pattern_init_solid (&tmp.solid, &combined,
+				   CAIRO_COLOR_IS_OPAQUE (&combined) ?
+				   CAIRO_CONTENT_COLOR :
+				   CAIRO_CONTENT_COLOR_ALPHA);
 
 	mask = NULL;
     } else {
-	_cairo_pattern_init_copy (&tmp.base, src);
+	status = _cairo_pattern_init_copy (&tmp.base, src);
+	if (status)
+	    return status;
     }
 
     status = _cairo_glitz_pattern_acquire_surface (&tmp.base, dst,
@@ -804,7 +846,9 @@ _cairo_glitz_pattern_acquire_surfaces (cairo_pattern_t	                *src,
 
     if (mask)
     {
-	_cairo_pattern_init_copy (&tmp.base, mask);
+	status = _cairo_pattern_init_copy (&tmp.base, mask);
+	if (status)
+	    return status;
 
 	status = _cairo_glitz_pattern_acquire_surface (&tmp.base, dst,
 						       mask_x, mask_y,
@@ -939,15 +983,25 @@ _cairo_glitz_surface_fill_rectangles (void		      *abstract_dst,
 			      (glitz_rectangle_t *) rects, n_rects);
     } break;
     case CAIRO_OPERATOR_CLEAR: {
-	static glitz_color_t glitz_color = { 0, 0, 0, 0 };
+	static const glitz_color_t glitz_color = { 0, 0, 0, 0 };
 
 	glitz_set_rectangles (dst->surface, &glitz_color,
 			      (glitz_rectangle_t *) rects, n_rects);
     } break;
+    case CAIRO_OPERATOR_SATURATE:
+	return CAIRO_INT_STATUS_UNSUPPORTED;
+    case CAIRO_OPERATOR_OVER:
+    case CAIRO_OPERATOR_IN:
+    case CAIRO_OPERATOR_OUT:
+    case CAIRO_OPERATOR_ATOP:
+    case CAIRO_OPERATOR_DEST:
+    case CAIRO_OPERATOR_DEST_OVER:
+    case CAIRO_OPERATOR_DEST_IN:
+    case CAIRO_OPERATOR_DEST_OUT:
+    case CAIRO_OPERATOR_DEST_ATOP:
+    case CAIRO_OPERATOR_XOR:
+    case CAIRO_OPERATOR_ADD:
     default:
-	if (op == CAIRO_OPERATOR_SATURATE)
-	    return CAIRO_INT_STATUS_UNSUPPORTED;
-
 	if (_glitz_ensure_target (dst->surface))
 	    return CAIRO_INT_STATUS_UNSUPPORTED;
 
@@ -955,7 +1009,8 @@ _cairo_glitz_surface_fill_rectangles (void		      *abstract_dst,
 	    _cairo_surface_create_similar_solid (&dst->base,
 						 CAIRO_CONTENT_COLOR_ALPHA,
 						 1, 1,
-						 (cairo_color_t *) color);
+						 (cairo_color_t *) color,
+						 NULL);
 	if (src->base.status)
 	    return CAIRO_STATUS_NO_MEMORY;
 
@@ -1024,7 +1079,9 @@ _cairo_glitz_surface_composite_trapezoids (cairo_operator_t  op,
 
     if (pattern->type == CAIRO_PATTERN_TYPE_SURFACE)
     {
-	_cairo_pattern_init_copy (&tmp_src_pattern.base, pattern);
+	status = _cairo_pattern_init_copy (&tmp_src_pattern.base, pattern);
+	if (status)
+	    return status;
 
 	status = _cairo_glitz_pattern_acquire_surface (&tmp_src_pattern.base,
 						       dst,
@@ -1048,7 +1105,7 @@ _cairo_glitz_surface_composite_trapezoids (cairo_operator_t  op,
 
     if (op == CAIRO_OPERATOR_ADD || n_traps <= 1)
     {
-	static glitz_color_t	clear_black = { 0, 0, 0, 0 };
+	static const glitz_color_t	clear_black = { 0, 0, 0, 0 };
 	glitz_color_t		color;
 	glitz_geometry_format_t	format;
 	int			n_trap_added;
@@ -1183,7 +1240,9 @@ _cairo_glitz_surface_composite_trapezoids (cairo_operator_t  op,
 	    return CAIRO_STATUS_NO_MEMORY;
 	}
 
-	_cairo_glitz_surface_set_image (mask, image, 0, 0);
+	_cairo_glitz_surface_set_image (mask, image, 0, 0, width, height, 0, 0);
+
+	cairo_surface_destroy(&image->base);
     }
 
     _cairo_glitz_surface_set_attributes (src, &attributes);
@@ -1234,26 +1293,32 @@ _cairo_glitz_surface_set_clip_region (void		*abstract_surface,
 	glitz_box_t *box;
 	int	    n;
 
-	if (!surface->clip)
-	{
-	    surface->clip = pixman_region_create ();
-	    if (!surface->clip)
-		return CAIRO_STATUS_NO_MEMORY;
-	}
-	pixman_region_copy (surface->clip, region);
+	if (!surface->has_clip) {
+            pixman_region_init (&surface->clip);
+            surface->has_clip = TRUE;
+        }
+
+	if (pixman_region_copy (&surface->clip, region) !=
+            PIXMAN_REGION_STATUS_SUCCESS)
+        {
+	    pixman_region_fini (&surface->clip);
+	    surface->has_clip = FALSE;
+            return CAIRO_STATUS_NO_MEMORY;
+        }
+
+	box = (glitz_box_t *) pixman_region_rects (&surface->clip);
+	n = pixman_region_num_rects (&surface->clip);
 
-	box = (glitz_box_t *) pixman_region_rects (surface->clip);
-	n = pixman_region_num_rects (surface->clip);
 	glitz_surface_set_clip_region (surface->surface, 0, 0, box, n);
     }
     else
     {
 	glitz_surface_set_clip_region (surface->surface, 0, 0, NULL, 0);
 
-	if (surface->clip)
-	    pixman_region_destroy (surface->clip);
-
-	surface->clip = NULL;
+	if (surface->has_clip) {
+	    pixman_region_fini (&surface->clip);
+	    surface->has_clip = FALSE;
+        }
     }
 
     return CAIRO_STATUS_SUCCESS;
@@ -1635,8 +1700,8 @@ static const cairo_glitz_area_funcs_t _cairo_glitz_area_funcs = {
 
 #define GLYPH_CACHE_TEXTURE_SIZE 512
 #define GLYPH_CACHE_MAX_LEVEL     64
-#define GLYPH_CACHE_MAX_HEIGHT    72
-#define GLYPH_CACHE_MAX_WIDTH     72
+#define GLYPH_CACHE_MAX_HEIGHT    96
+#define GLYPH_CACHE_MAX_WIDTH     96
 
 #define WRITE_VEC2(ptr, _x, _y) \
     *(ptr)++ = (_x);		\
@@ -1665,10 +1730,14 @@ _cairo_glitz_surface_font_init (cairo_glitz_surface_t *surface,
     drawable = glitz_surface_get_drawable (surface->surface);
 
     switch (format) {
+    case CAIRO_FORMAT_A1:
     case CAIRO_FORMAT_A8:
 	surface_format =
 	    glitz_find_standard_format (drawable, GLITZ_STANDARD_A8);
 	break;
+    case CAIRO_FORMAT_RGB24:
+	ASSERT_NOT_REACHED;
+	break;
     case CAIRO_FORMAT_ARGB32:
 	surface_format =
 	    glitz_find_standard_format (drawable, GLITZ_STANDARD_ARGB32);
@@ -1760,7 +1829,7 @@ _cairo_glitz_surface_add_glyph (cairo_glitz_surface_t *surface,
     glitz_pixel_format_t		pf;
     glitz_buffer_t			*buffer;
     pixman_format_t			*format;
-    int					am, rm, gm, bm;
+    unsigned int			bpp, am, rm, gm, bm;
     cairo_int_status_t			status;
 
     glyph_private = scaled_glyph->surface_private;
@@ -1819,9 +1888,10 @@ _cairo_glitz_surface_add_glyph (cairo_glitz_surface_t *surface,
 	return CAIRO_STATUS_NO_MEMORY;
     }
 
-    pixman_format_get_masks (format, &pf.masks.bpp, &am, &rm, &gm, &bm);
+    pixman_format_get_masks (format, &bpp, &am, &rm, &gm, &bm);
 
     pf.fourcc		= GLITZ_FOURCC_RGB;
+    pf.masks.bpp        = bpp;
     pf.masks.alpha_mask = am;
     pf.masks.red_mask   = rm;
     pf.masks.green_mask = gm;
@@ -1870,7 +1940,7 @@ _cairo_glitz_surface_old_show_glyphs (cairo_scaled_font_t *scaled_font,
 				      int		   dst_y,
 				      unsigned int	   width,
 				      unsigned int	   height,
-				      const cairo_glyph_t *glyphs,
+				      cairo_glyph_t       *glyphs,
 				      int		   num_glyphs)
 {
     cairo_glitz_surface_attributes_t	attributes;
@@ -1887,7 +1957,7 @@ _cairo_glitz_surface_old_show_glyphs (cairo_scaled_font_t *scaled_font,
     int					i, cached_glyphs = 0;
     int					remaining_glyps = num_glyphs;
     glitz_float_t			x1, y1, x2, y2;
-    static glitz_vertex_format_t	format = {
+    static const glitz_vertex_format_t	format = {
 	GLITZ_PRIMITIVE_QUADS,
 	GLITZ_DATA_TYPE_FLOAT,
 	sizeof (glitz_float_t) * 4,
@@ -1941,6 +2011,8 @@ _cairo_glitz_surface_old_show_glyphs (cairo_scaled_font_t *scaled_font,
     if (!buffer)
 	goto FAIL2;
 
+    _cairo_scaled_font_freeze_cache (scaled_font);
+
     for (i = 0; i < num_glyphs; i++)
     {
 	status = _cairo_scaled_glyph_lookup (scaled_font,
@@ -1953,6 +2025,17 @@ _cairo_glitz_surface_old_show_glyphs (cairo_scaled_font_t *scaled_font,
 	    goto UNLOCK;
 	}
 
+	glyph_private = scaled_glyphs[i]->surface_private;
+	if (!glyph_private || !glyph_private->area)
+	{
+	    status = _cairo_glitz_surface_add_glyph (dst,
+						     scaled_font,
+						     scaled_glyphs[i]);
+	    if (status != CAIRO_STATUS_SUCCESS) {
+		num_glyphs = i;
+		goto UNLOCK;
+	    }
+	}
 	glyph_private = scaled_glyphs[i]->surface_private;
 	if (glyph_private && glyph_private->area)
 	{
@@ -1963,8 +2046,8 @@ _cairo_glitz_surface_old_show_glyphs (cairo_scaled_font_t *scaled_font,
 		x_offset = scaled_glyphs[i]->surface->base.device_transform.x0;
 		y_offset = scaled_glyphs[i]->surface->base.device_transform.y0;
 
-		x1 = floor (glyphs[i].x + 0.5) + x_offset;
-		y1 = floor (glyphs[i].y + 0.5) + y_offset;
+		x1 = _cairo_lround (glyphs[i].x) + x_offset;
+		y1 = _cairo_lround (glyphs[i].y) + y_offset;
 		x2 = x1 + glyph_private->area->width;
 		y2 = y1 + glyph_private->area->height;
 
@@ -1988,47 +2071,28 @@ _cairo_glitz_surface_old_show_glyphs (cairo_scaled_font_t *scaled_font,
 	    glyph_private = scaled_glyphs[i]->surface_private;
 	    if (!glyph_private || !glyph_private->area)
 	    {
-		status = _cairo_glitz_surface_add_glyph (dst,
-							 scaled_font,
-							 scaled_glyphs[i]);
-		if (status)
-		    goto UNLOCK;
+		int glyph_width, glyph_height;
 
-		glyph_private = scaled_glyphs[i]->surface_private;
-	    }
-
-	    x_offset = scaled_glyphs[i]->surface->base.device_transform.x0;
-	    y_offset = scaled_glyphs[i]->surface->base.device_transform.y0;
-
-	    x1 = floor (glyphs[i].x + 0.5) + x_offset;
-	    y1 = floor (glyphs[i].y + 0.5) + y_offset;
-
-	    if (glyph_private->area)
-	    {
-		if (glyph_private->area->width)
-		{
-		    x2 = x1 + glyph_private->area->width;
-		    y2 = y1 + glyph_private->area->height;
-
-		    WRITE_BOX (vertices, x1, y1, x2, y2,
-			       &glyph_private->p1, &glyph_private->p2);
-
-		    glyph_private->locked = TRUE;
-
-		    cached_glyphs++;
-		}
-	    }
-	    else
-	    {
 		image = &scaled_glyphs[i]->surface->base;
+		glyph_width = scaled_glyphs[i]->surface->width;
+		glyph_height = scaled_glyphs[i]->surface->height;
 		status =
 		    _cairo_glitz_surface_clone_similar (abstract_surface,
 							image,
+							0,
+							0,
+							glyph_width,
+							glyph_height,
 							(cairo_surface_t **)
 							&clone);
 		if (status)
 		    goto UNLOCK;
 
+		x_offset = scaled_glyphs[i]->surface->base.device_transform.x0;
+		y_offset = scaled_glyphs[i]->surface->base.device_transform.y0;
+		x1 = _cairo_lround (glyphs[i].x) + x_offset;
+		y1 = _cairo_lround (glyphs[i].y) + y_offset;
+
 		glitz_composite (_glitz_operator (op),
 				 src->surface,
 				 clone->surface,
@@ -2037,8 +2101,8 @@ _cairo_glitz_surface_old_show_glyphs (cairo_scaled_font_t *scaled_font,
 				 src_y + attributes.base.y_offset + y1,
 				 0, 0,
 				 x1, y1,
-				 scaled_glyphs[i]->surface->width,
-				 scaled_glyphs[i]->surface->height);
+				 glyph_width,
+				 glyph_height);
 
 		cairo_surface_destroy (&clone->base);
 
@@ -2091,6 +2155,8 @@ UNLOCK:
 	}
     }
 
+    _cairo_scaled_font_thaw_cache (scaled_font);
+
     glitz_buffer_destroy (buffer);
 
  FAIL2:
@@ -2122,6 +2188,33 @@ _cairo_glitz_surface_flush (void *abstract_surface)
     return CAIRO_STATUS_SUCCESS;
 }
 
+static cairo_bool_t
+_cairo_glitz_surface_is_similar (void *surface_a,
+	                         void *surface_b,
+				 cairo_content_t content)
+{
+    cairo_glitz_surface_t *a = (cairo_glitz_surface_t *) surface_a;
+    cairo_glitz_surface_t *b = (cairo_glitz_surface_t *) surface_b;
+
+    glitz_drawable_t *drawable_a = glitz_surface_get_drawable (a->surface);
+    glitz_drawable_t *drawable_b = glitz_surface_get_drawable (b->surface);
+
+    return drawable_a == drawable_b;
+}
+
+static cairo_status_t
+_cairo_glitz_surface_reset (void *abstract_surface)
+{
+    cairo_glitz_surface_t *surface = abstract_surface;
+    cairo_status_t status;
+
+    status = _cairo_glitz_surface_set_clip_region (surface, NULL);
+    if (status)
+	return status;
+
+    return CAIRO_STATUS_SUCCESS;
+}
+
 static const cairo_surface_backend_t cairo_glitz_surface_backend = {
     CAIRO_SURFACE_TYPE_GLITZ,
     _cairo_glitz_surface_create_similar,
@@ -2144,7 +2237,18 @@ static const cairo_surface_backend_t cairo_glitz_surface_backend = {
     _cairo_glitz_surface_flush,
     NULL, /* mark_dirty_rectangle */
     _cairo_glitz_surface_scaled_font_fini,
-    _cairo_glitz_surface_scaled_glyph_fini
+    _cairo_glitz_surface_scaled_glyph_fini,
+
+    NULL, /* paint */
+    NULL, /* mask */
+    NULL, /* stroke */
+    NULL, /* fill */
+    NULL, /* show_glyphs */
+
+    NULL, /* snapshot */
+    _cairo_glitz_surface_is_similar,
+
+    _cairo_glitz_surface_reset
 };
 
 static const cairo_surface_backend_t *
@@ -2190,9 +2294,10 @@ cairo_glitz_surface_create (glitz_surface_t *surface)
 
     glitz_surface_reference (surface);
 
-    crsurface->surface = surface;
-    crsurface->format  = format;
-    crsurface->clip    = NULL;
+    crsurface->surface  = surface;
+    crsurface->format   = format;
+    crsurface->has_clip = FALSE;
 
     return (cairo_surface_t *) crsurface;
 }
+slim_hidden_def (cairo_glitz_surface_create);
